@@ -161,4 +161,7 @@ class MongoClientImpl implements MongoClient {
         return cluster;
     }
 
+    ServerSessionPool getServerSessionPool() {
+        return serverSessionPool;
+    }
 }
