@@ -54,7 +54,22 @@ public final class Filters {
     }
 
     /**
-     * Creates a filter that matches all documents where the value of the field name equals the specified value. Note that this does
+     * Creates a filter that matches all documents where the value of _id field equals the specified value. Note that this doesn't
+     * actually generate a $eq operator, as the query language doesn't require it.
+     *
+     * @param value     the value
+     * @param <TItem>   the value type
+     * @return the filter
+     * @mongodb.driver.manual reference/operator/query/eq $eq
+     *
+     * @since 3.4
+     */
+    public static <TItem> Bson eq(final TItem value) {
+        return eq("_id", value);
+    }
+
+    /**
+     * Creates a filter that matches all documents where the value of the field name equals the specified value. Note that this doesn't
      * actually generate a $eq operator, as the query language doesn't require it.
      *
      * @param fieldName the field name
@@ -453,6 +468,20 @@ public final class Filters {
         return new BsonDocument("$where", new BsonString(javaScriptExpression));
     }
 
+    /**
+     * Creates a filter that matches all documents that validate against the given JSON schema document.
+     *
+     * @param expression the aggregation expression
+     * @param <TExpression> the expression type
+     * @return the filter
+     * @since 3.6
+     * @mongodb.server.release 3.6
+     * @mongodb.driver.manual reference/operator/query/expr/ $expr
+     */
+    public static <TExpression> Bson expr(final TExpression expression) {
+        return new SimpleEncodingFilter<TExpression>("$expr", expression);
+    }
+
     /**
      * Creates a filter that matches all documents where the value of a field is an array that contains all the specified values.
      *
@@ -813,6 +842,19 @@ public final class Filters {
         return createNearFilterDocument(fieldName, x, y, maxDistance, minDistance, "$nearSphere");
     }
 
+    /**
+     * Creates a filter that matches all documents that validate against the given JSON schema document.
+     *
+     * @param schema the JSON schema to validate against
+     * @return the filter
+     * @since 3.6
+     * @mongodb.server.release 3.6
+     * @mongodb.driver.manual reference/operator/query/jsonSchema/ $jsonSchema
+     */
+    public static Bson jsonSchema(final Bson schema) {
+        return new SimpleEncodingFilter<Bson>("$jsonSchema", schema);
+    }
+
     private static Bson createNearFilterDocument(final String fieldName, final double x, final double y, final Double maxDistance,
                                                  final Double minDistance, final String operator) {
         BsonDocument nearFilter = new BsonDocument(operator, new BsonArray(Arrays.asList(new BsonDouble(x), new BsonDouble(y))));
@@ -888,7 +930,7 @@ public final class Filters {
     private static class AndFilter implements Bson {
         private final Iterable<Bson> filters;
 
-        public AndFilter(final Iterable<Bson> filters) {
+        AndFilter(final Iterable<Bson> filters) {
             this.filters = notNull("filters", filters);
         }
 
@@ -980,7 +1022,7 @@ public final class Filters {
         private final Operator operator;
         private final Iterable<Bson> filters;
 
-        public OrNorFilter(final Operator operator, final Iterable<Bson> filters) {
+        OrNorFilter(final Operator operator, final Iterable<Bson> filters) {
             this.operator = notNull("operator", operator);
             this.filters = notNull("filters", filters);
         }
@@ -1049,7 +1091,7 @@ public final class Filters {
         private final String fieldName;
         private final TItem value;
 
-        public SimpleEncodingFilter(final String fieldName, final TItem value) {
+        SimpleEncodingFilter(final String fieldName, final TItem value) {
             this.fieldName = notNull("fieldName", fieldName);
             this.value = value;
         }
@@ -1078,7 +1120,7 @@ public final class Filters {
     private static class NotFilter implements Bson {
         private final Bson filter;
 
-        public NotFilter(final Bson filter) {
+        NotFilter(final Bson filter) {
             this.filter = notNull("filter", filter);
         }
 
@@ -1121,12 +1163,12 @@ public final class Filters {
         private final Double maxDistance;
         private final Double minDistance;
 
-        public GeometryOperatorFilter(final String operatorName, final String fieldName, final TItem geometry) {
+        GeometryOperatorFilter(final String operatorName, final String fieldName, final TItem geometry) {
             this(operatorName, fieldName, geometry, null, null);
         }
 
-        public GeometryOperatorFilter(final String operatorName, final String fieldName, final TItem geometry,
-                                      final Double maxDistance, final Double minDistance) {
+        GeometryOperatorFilter(final String operatorName, final String fieldName, final TItem geometry,
+                               final Double maxDistance, final Double minDistance) {
             this.operatorName = operatorName;
             this.fieldName = notNull("fieldName", fieldName);
             this.geometry = notNull("geometry", geometry);
@@ -1173,7 +1215,7 @@ public final class Filters {
         private final String search;
         private final TextSearchOptions textSearchOptions;
 
-        public TextFilter(final String search, final TextSearchOptions textSearchOptions) {
+        TextFilter(final String search, final TextSearchOptions textSearchOptions) {
             this.search = search;
             this.textSearchOptions = textSearchOptions;
         }
