@@ -33,10 +33,12 @@ import com.mongodb.binding.AsyncSingleConnectionReadBinding;
 import com.mongodb.binding.ConnectionSource;
 import com.mongodb.binding.ReadBinding;
 import com.mongodb.binding.SingleConnectionReadBinding;
+import com.mongodb.client.model.Collation;
 import com.mongodb.connection.AsyncConnection;
 import com.mongodb.connection.Connection;
 import com.mongodb.connection.ConnectionDescription;
 import com.mongodb.connection.QueryResult;
+import com.mongodb.session.SessionContext;
 import com.mongodb.operation.OperationHelper.CallableWithConnectionAndSource;
 import org.bson.BsonBoolean;
 import org.bson.BsonDocument;
@@ -65,11 +67,12 @@ import static com.mongodb.operation.CommandOperationHelper.executeWrappedCommand
 import static com.mongodb.operation.CommandOperationHelper.executeWrappedCommandProtocolAsync;
 import static com.mongodb.operation.OperationHelper.AsyncCallableWithConnectionAndSource;
 import static com.mongodb.operation.OperationHelper.LOGGER;
-import static com.mongodb.operation.OperationHelper.checkValidReadConcern;
 import static com.mongodb.operation.OperationHelper.cursorDocumentToQueryResult;
 import static com.mongodb.operation.OperationHelper.releasingCallback;
 import static com.mongodb.operation.OperationHelper.serverIsAtLeastVersionThreeDotTwo;
+import static com.mongodb.operation.OperationHelper.validateReadConcernAndCollation;
 import static com.mongodb.operation.OperationHelper.withConnection;
+import static com.mongodb.operation.ReadConcernHelper.appendReadConcernToCommand;
 
 /**
  * An operation that queries a collection using the provided criteria.
@@ -97,6 +100,15 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
     private boolean noCursorTimeout;
     private boolean partial;
     private ReadConcern readConcern = ReadConcern.DEFAULT;
+    private Collation collation;
+    private String comment;
+    private BsonDocument hint;
+    private BsonDocument max;
+    private BsonDocument min;
+    private long maxScan;
+    private boolean returnKey;
+    private boolean showRecordId;
+    private boolean snapshot;
 
     /**
      * Construct a new instance.
@@ -209,7 +221,9 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
      * @param modifiers the query modifiers to apply, which may be null.
      * @return this
      * @mongodb.driver.manual reference/operator/query-modifier/ Query Modifiers
+     * @deprecated use the individual setters instead
      */
+    @Deprecated
     public FindOperation<T> modifiers(final BsonDocument modifiers) {
         this.modifiers = modifiers;
         return this;
@@ -478,6 +492,220 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
         return this;
     }
 
+    /**
+     * Returns the collation options
+     *
+     * @return the collation options
+     * @since 3.4
+     * @mongodb.server.release 3.4
+     */
+    public Collation getCollation() {
+        return collation;
+    }
+
+    /**
+     * Sets the collation options
+     *
+     * <p>A null value represents the server default.</p>
+     * @param collation the collation options to use
+     * @return this
+     * @since 3.4
+     * @mongodb.server.release 3.4
+     */
+    public FindOperation<T> collation(final Collation collation) {
+        this.collation = collation;
+        return this;
+    }
+
+    /**
+     * Returns the comment to send with the query. The default is not to include a comment with the query.
+     *
+     * @return the comment
+     * @since 3.5
+     */
+    public String getComment() {
+        return comment;
+    }
+
+    /**
+     * Sets the comment to the query. A null value means no comment is set.
+     *
+     * @param comment the comment
+     * @return this
+     * @since 3.5
+     */
+    public FindOperation<T> comment(final String comment) {
+        this.comment = comment;
+        return this;
+    }
+
+    /**
+     * Returns the hint for which index to use. The default is not to set a hint.
+     *
+     * @return the hint
+     * @since 3.5
+     */
+    public BsonDocument getHint() {
+        return hint;
+    }
+
+    /**
+     * Sets the hint for which index to use. A null value means no hint is set.
+     *
+     * @param hint the hint
+     * @return this
+     * @since 3.5
+     */
+    public FindOperation<T> hint(final BsonDocument hint) {
+        this.hint = hint;
+        return this;
+    }
+
+    /**
+     * Returns the exclusive upper bound for a specific index. By default there is no max bound.
+     *
+     * @return the max
+     * @since 3.5
+     */
+    public BsonDocument getMax() {
+        return max;
+    }
+
+    /**
+     * Sets the exclusive upper bound for a specific index. A null value means no max is set.
+     *
+     * @param max the max
+     * @return this
+     * @since 3.5
+     */
+    public FindOperation<T> max(final BsonDocument max) {
+        this.max = max;
+        return this;
+    }
+
+    /**
+     * Returns the minimum inclusive lower bound for a specific index. By default there is no min bound.
+     *
+     * @return the min
+     * @since 3.5
+     */
+    public BsonDocument getMin() {
+        return min;
+    }
+
+    /**
+     * Sets the minimum inclusive lower bound for a specific index. A null value means no max is set.
+     *
+     * @param min the min
+     * @return this
+     * @since 3.5
+     */
+    public FindOperation<T> min(final BsonDocument min) {
+        this.min = min;
+        return this;
+    }
+
+    /**
+     * Returns the maximum number of documents or index keys to scan when executing the query.
+     *
+     * A zero value or less will be ignored, and indicates that the driver should respect the server's default value.
+     *
+     * @return the maxScan
+     * @since 3.5
+     */
+    public long getMaxScan() {
+        return maxScan;
+    }
+
+    /**
+     * Sets the maximum number of documents or index keys to scan when executing the query.
+     *
+     * A zero value or less will be ignored, and indicates that the driver should respect the server's default value.
+     *
+     * @param maxScan the maxScan
+     * @return this
+     * @since 3.5
+     */
+    public FindOperation<T> maxScan(final long maxScan) {
+        this.maxScan = maxScan;
+        return this;
+    }
+
+    /**
+     * Returns the returnKey. If true the find operation will return only the index keys in the resulting documents.
+     *
+     * Default value is false. If returnKey is true and the find command does not use an index, the returned documents will be empty.
+     *
+     * @return the returnKey
+     * @since 3.5
+     */
+    public boolean isReturnKey() {
+        return returnKey;
+    }
+
+    /**
+     * Sets the returnKey. If true the find operation will return only the index keys in the resulting documents.
+     *
+     * @param returnKey the returnKey
+     * @return this
+     * @since 3.5
+     */
+    public FindOperation<T> returnKey(final boolean returnKey) {
+        this.returnKey = returnKey;
+        return this;
+    }
+
+    /**
+     * Returns the showRecordId.
+     *
+     * Determines whether to return the record identifier for each document. If true, adds a field $recordId to the returned documents.
+     * The default is false.
+     *
+     * @return the showRecordId
+     * @since 3.5
+     */
+    public boolean isShowRecordId() {
+        return showRecordId;
+    }
+
+    /**
+     * Sets the showRecordId. Set to true to add a field {@code $recordId} to the returned documents.
+     *
+     * @param showRecordId the showRecordId
+     * @return this
+     * @since 3.5
+     */
+    public FindOperation<T> showRecordId(final boolean showRecordId) {
+        this.showRecordId = showRecordId;
+        return this;
+    }
+
+    /**
+     * Returns the snapshot.
+     *
+     * Prevents the cursor from returning a document more than once because of an intervening write operation. The default is false.
+     *
+     * @return the snapshot
+     * @since 3.5
+     */
+    public boolean isSnapshot() {
+        return snapshot;
+    }
+
+    /**
+     * Sets the snapshot.
+     *
+     * If true it prevents the cursor from returning a document more than once because of an intervening write operation.
+     *
+     * @param snapshot the snapshot
+     * @return this
+     * @since 3.5
+     */
+    public FindOperation<T> snapshot(final boolean snapshot) {
+        this.snapshot = snapshot;
+        return this;
+    }
+
     @Override
     public BatchCursor<T> execute(final ReadBinding binding) {
         return withConnection(binding, new CallableWithConnectionAndSource<BatchCursor<T>>() {
@@ -485,15 +713,16 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
             public BatchCursor<T> call(final ConnectionSource source, final Connection connection) {
                 if (serverIsAtLeastVersionThreeDotTwo(connection.getDescription())) {
                     try {
+                        validateReadConcernAndCollation(connection, readConcern, collation);
                         return executeWrappedCommandProtocol(binding, namespace.getDatabaseName(),
-                                                             wrapInExplainIfNecessary(asCommandDocument()),
+                                                             wrapInExplainIfNecessary(getCommand(binding.getSessionContext())),
                                                              CommandResultDocumentCodec.create(decoder, FIRST_BATCH),
                                                              connection, transformer(source, connection));
                     } catch (MongoCommandException e) {
                         throw new MongoQueryException(e.getServerAddress(), e.getErrorCode(), e.getErrorMessage());
                     }
                 } else {
-                    checkValidReadConcern(connection, readConcern);
+                    validateReadConcernAndCollation(connection, readConcern, collation);
                     QueryResult<T> queryResult = connection.query(namespace,
                                                                   asDocument(connection.getDescription(), binding.getReadPreference()),
                                                                   projection,
@@ -523,39 +752,52 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
                     errHandlingCallback.onResult(null, t);
                 } else {
                     if (serverIsAtLeastVersionThreeDotTwo(connection.getDescription())) {
-                        executeWrappedCommandProtocolAsync(binding, namespace.getDatabaseName(),
-                                                           wrapInExplainIfNecessary(asCommandDocument()),
-                                                           CommandResultDocumentCodec.create(decoder, FIRST_BATCH),
-                                                           connection, asyncTransformer(source, connection),
-                                                           releasingCallback(exceptionTransformingCallback(errHandlingCallback),
-                                                                             source, connection));
+                        final SingleResultCallback<AsyncBatchCursor<T>> wrappedCallback =
+                                releasingCallback(exceptionTransformingCallback(errHandlingCallback), source, connection);
+                        validateReadConcernAndCollation(source, connection, readConcern, collation,
+                                new AsyncCallableWithConnectionAndSource() {
+                                    @Override
+                                    public void call(final AsyncConnectionSource source, final AsyncConnection connection,
+                                                     final Throwable t) {
+                                        if (t != null) {
+                                            wrappedCallback.onResult(null, t);
+                                        } else {
+                                            executeWrappedCommandProtocolAsync(binding, namespace.getDatabaseName(),
+                                                    wrapInExplainIfNecessary(getCommand(binding.getSessionContext())),
+                                                    CommandResultDocumentCodec.create(decoder, FIRST_BATCH), connection,
+                                                    asyncTransformer(source, connection), wrappedCallback);
+                                        }
+                                    }
+                                });
                     } else {
-                        final SingleResultCallback<AsyncBatchCursor<T>> wrappedCallback = releasingCallback(errHandlingCallback,
-
-                                source, connection);
-                        checkValidReadConcern(source, connection, readConcern, new AsyncCallableWithConnectionAndSource() {
-                            @Override
-                            public void call(final AsyncConnectionSource source, final AsyncConnection connection, final Throwable t) {
-                                if (t != null) {
-                                    wrappedCallback.onResult(null, t);
-                                } else {
-                                    connection.queryAsync(namespace, asDocument(connection.getDescription(), binding.getReadPreference()),
-                                            projection, skip, limit, batchSize, isSlaveOk() || binding.getReadPreference().isSlaveOk(),
-                                            isTailableCursor(), isAwaitData(), isNoCursorTimeout(), isPartial(), isOplogReplay(), decoder,
-                                            new SingleResultCallback<QueryResult<T>>() {
-                                                @Override
-                                                public void onResult(final QueryResult<T> result, final Throwable t) {
-                                                    if (t != null) {
-                                                        wrappedCallback.onResult(null, t);
-                                                    } else {
-                                                        wrappedCallback.onResult(new AsyncQueryBatchCursor<T>(result, limit, batchSize,
-                                                                        getMaxTimeForCursor(),
-                                                                        decoder, source, connection), null);
-                                                    }
-                                                }
-                                            });
-                                }
-                            }
+                        final SingleResultCallback<AsyncBatchCursor<T>> wrappedCallback =
+                                releasingCallback(errHandlingCallback, source, connection);
+                        validateReadConcernAndCollation(source, connection, readConcern, collation,
+                                new AsyncCallableWithConnectionAndSource() {
+                                    @Override
+                                    public void call(final AsyncConnectionSource source, final AsyncConnection connection, final
+                                    Throwable t) {
+                                        if (t != null) {
+                                            wrappedCallback.onResult(null, t);
+                                        } else {
+                                            connection.queryAsync(namespace, asDocument(connection.getDescription(),
+                                                    binding.getReadPreference()), projection, skip, limit, batchSize,
+                                                    isSlaveOk() || binding.getReadPreference().isSlaveOk(),
+                                                    isTailableCursor(), isAwaitData(), isNoCursorTimeout(), isPartial(), isOplogReplay(),
+                                                    decoder, new SingleResultCallback<QueryResult<T>>() {
+                                                        @Override
+                                                        public void onResult(final QueryResult<T> result, final Throwable t) {
+                                                            if (t != null) {
+                                                                wrappedCallback.onResult(null, t);
+                                                            } else {
+                                                                wrappedCallback.onResult(new AsyncQueryBatchCursor<T>(result, limit,
+                                                                        batchSize, getMaxTimeForCursor(), decoder, source, connection),
+                                                                        null);
+                                                            }
+                                                        }
+                                                    });
+                                        }
+                                    }
                         });
                     }
                 }
@@ -605,7 +847,8 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
                             if (serverIsAtLeastVersionThreeDotTwo(connection.getDescription())) {
                                 try {
                                     return new CommandReadOperation<BsonDocument>(getNamespace().getDatabaseName(),
-                                                                                  new BsonDocument("explain", asCommandDocument()),
+                                                                                  new BsonDocument("explain",
+                                                                                                   getCommand(binding.getSessionContext())),
                                                                                   new BsonDocumentCodec()).execute(singleConnectionBinding);
                                 } catch (MongoCommandException e) {
                                     throw new MongoQueryException(e.getServerAddress(), e.getErrorCode(), e.getErrorMessage());
@@ -651,7 +894,7 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
 
                             if (serverIsAtLeastVersionThreeDotTwo(connection.getDescription())) {
                                 new CommandReadOperation<BsonDocument>(namespace.getDatabaseName(),
-                                                                       new BsonDocument("explain", asCommandDocument()),
+                                                                       new BsonDocument("explain", getCommand(binding.getSessionContext())),
                                                                        new BsonDocumentCodec())
                                 .executeAsync(singleConnectionReadBinding,
                                               releasingCallback(exceptionTransformingCallback(errHandlingCallback),
@@ -693,18 +936,39 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
         if (modifiers != null) {
             document.putAll(modifiers);
         }
-
         if (sort != null) {
             document.put("$orderby", sort);
         }
-
         if (maxTimeMS > 0) {
             document.put("$maxTimeMS", new BsonInt64(maxTimeMS));
         }
-
         if (connectionDescription.getServerType() == SHARD_ROUTER && !readPreference.equals(primary())) {
             document.put("$readPreference", readPreference.toDocument());
         }
+        if (comment != null) {
+            document.put("$comment", new BsonString(comment));
+        }
+        if (hint != null) {
+            document.put("$hint", hint);
+        }
+        if (max != null) {
+            document.put("$max", max);
+        }
+        if (min != null) {
+            document.put("$min", min);
+        }
+        if (maxScan > 0) {
+            document.put("$maxScan", new BsonInt64(maxScan));
+        }
+        if (returnKey) {
+            document.put("$returnKey", BsonBoolean.TRUE);
+        }
+        if (showRecordId) {
+            document.put("$showDiskLoc", BsonBoolean.TRUE);
+        }
+        if (snapshot) {
+            document.put("$snapshot", BsonBoolean.TRUE);
+        }
 
         if (document.isEmpty()) {
             document = filter != null ? filter : new BsonDocument();
@@ -733,86 +997,97 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
         META_OPERATOR_TO_COMMAND_FIELD_MAP.put("$snapshot", "snapshot");
     }
 
-    private BsonDocument asCommandDocument() {
-        BsonDocument document = new BsonDocument("find", new BsonString(namespace.getCollectionName()));
+    private BsonDocument getCommand(final SessionContext sessionContext) {
+        BsonDocument commandDocument = new BsonDocument("find", new BsonString(namespace.getCollectionName()));
+
+        appendReadConcernToCommand(readConcern, sessionContext, commandDocument);
 
         if (modifiers != null) {
             for (Map.Entry<String, BsonValue> cur : modifiers.entrySet()) {
                 String commandFieldName = META_OPERATOR_TO_COMMAND_FIELD_MAP.get(cur.getKey());
                 if (commandFieldName != null) {
-                    document.append(commandFieldName, cur.getValue());
+                    commandDocument.append(commandFieldName, cur.getValue());
                 }
             }
         }
-
         if (filter != null) {
-            document.put("filter", filter);
+            commandDocument.put("filter", filter);
         }
-
         if (sort != null) {
-            document.put("sort", sort);
+            commandDocument.put("sort", sort);
         }
-
         if (projection != null) {
-            document.put("projection", projection);
+            commandDocument.put("projection", projection);
         }
-
         if (skip > 0) {
-            document.put("skip", new BsonInt32(skip));
+            commandDocument.put("skip", new BsonInt32(skip));
         }
-
         if (limit != 0) {
-            document.put("limit", new BsonInt32(Math.abs(limit)));
+            commandDocument.put("limit", new BsonInt32(Math.abs(limit)));
         }
-
         if (limit >= 0) {
             if (batchSize < 0 && Math.abs(batchSize) < limit) {
-                document.put("limit", new BsonInt32(Math.abs(batchSize)));
+                commandDocument.put("limit", new BsonInt32(Math.abs(batchSize)));
             } else if (batchSize != 0) {
-                document.put("batchSize", new BsonInt32(Math.abs(batchSize)));
+                commandDocument.put("batchSize", new BsonInt32(Math.abs(batchSize)));
             }
         }
-
         if (limit < 0 || batchSize < 0) {
-            document.put("singleBatch", BsonBoolean.TRUE);
+            commandDocument.put("singleBatch", BsonBoolean.TRUE);
         }
-
         if (maxTimeMS > 0) {
-            document.put("maxTimeMS", new BsonInt64(maxTimeMS));
+            commandDocument.put("maxTimeMS", new BsonInt64(maxTimeMS));
         }
-
         if (isTailableCursor()) {
-            document.put("tailable", BsonBoolean.TRUE);
+            commandDocument.put("tailable", BsonBoolean.TRUE);
         }
-
         if (isAwaitData()) {
-            document.put("awaitData", BsonBoolean.TRUE);
+            commandDocument.put("awaitData", BsonBoolean.TRUE);
         }
-
         if (oplogReplay) {
-            document.put("oplogReplay", BsonBoolean.TRUE);
+            commandDocument.put("oplogReplay", BsonBoolean.TRUE);
         }
-
         if (noCursorTimeout) {
-            document.put("noCursorTimeout", BsonBoolean.TRUE);
+            commandDocument.put("noCursorTimeout", BsonBoolean.TRUE);
         }
-
         if (partial) {
-            document.put("allowPartialResults", BsonBoolean.TRUE);
+            commandDocument.put("allowPartialResults", BsonBoolean.TRUE);
         }
-
-        if (!readConcern.isServerDefault()) {
-            document.put("readConcern", readConcern.asDocument());
+        if (collation != null) {
+            commandDocument.put("collation", collation.asDocument());
         }
-
-        return document;
+        if (comment != null) {
+            commandDocument.put("comment", new BsonString(comment));
+        }
+        if (hint != null) {
+            commandDocument.put("hint", hint);
+        }
+        if (max != null) {
+            commandDocument.put("max", max);
+        }
+        if (min != null) {
+            commandDocument.put("min", min);
+        }
+        if (maxScan > 0) {
+            commandDocument.put("maxScan", new BsonInt64(maxScan));
+        }
+        if (returnKey) {
+            commandDocument.put("returnKey", BsonBoolean.TRUE);
+        }
+        if (showRecordId) {
+            commandDocument.put("showRecordId", BsonBoolean.TRUE);
+        }
+        if (snapshot) {
+            commandDocument.put("snapshot", BsonBoolean.TRUE);
+        }
+        return commandDocument;
     }
 
-    private BsonDocument wrapInExplainIfNecessary(final BsonDocument findCommandDocument) {
+    private BsonDocument wrapInExplainIfNecessary(final BsonDocument commandDocument) {
         if (isExplain()) {
-            return new BsonDocument("explain", findCommandDocument);
+            return new BsonDocument("explain", commandDocument);
         } else {
-            return findCommandDocument;
+            return commandDocument;
         }
     }
 
@@ -868,7 +1143,7 @@ public class FindOperation<T> implements AsyncReadOperation<AsyncBatchCursor<T>>
     private static class ExplainResultCallback implements SingleResultCallback<AsyncBatchCursor<BsonDocument>> {
         private final SingleResultCallback<BsonDocument> callback;
 
-        public ExplainResultCallback(final SingleResultCallback<BsonDocument> callback) {
+        ExplainResultCallback(final SingleResultCallback<BsonDocument> callback) {
             this.callback = callback;
         }
 
